use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::NoTabs 0.15

use Test::More 0.88;
use Test::NoTabs;

my @files = (
    'lib/XML/Grammar/Fiction.pm',
    'lib/XML/Grammar/Fiction/App/FromProto.pm',
    'lib/XML/Grammar/Fiction/App/ToDocBook.pm',
    'lib/XML/Grammar/Fiction/App/ToHTML.pm',
    'lib/XML/Grammar/Fiction/Err.pm',
    'lib/XML/Grammar/Fiction/FromProto.pm',
    'lib/XML/Grammar/Fiction/FromProto/Node.pm',
    'lib/XML/Grammar/Fiction/FromProto/Node/Comment.pm',
    'lib/XML/Grammar/Fiction/FromProto/Node/Description.pm',
    'lib/XML/Grammar/Fiction/FromProto/Node/Element.pm',
    'lib/XML/Grammar/Fiction/FromProto/Node/InnerDesc.pm',
    'lib/XML/Grammar/Fiction/FromProto/Node/List.pm',
    'lib/XML/Grammar/Fiction/FromProto/Node/Paragraph.pm',
    'lib/XML/Grammar/Fiction/FromProto/Node/Saying.pm',
    'lib/XML/Grammar/Fiction/FromProto/Node/Text.pm',
    'lib/XML/Grammar/Fiction/FromProto/Node/WithContent.pm',
    'lib/XML/Grammar/Fiction/FromProto/Parser.pm',
    'lib/XML/Grammar/Fiction/FromProto/Parser/QnD.pm',
    'lib/XML/Grammar/Fiction/Struct/Tag.pm',
    'lib/XML/Grammar/Fiction/ToDocBook.pm',
    'lib/XML/Grammar/Fiction/ToHTML.pm',
    'lib/XML/Grammar/FictionBase/Event.pm',
    'lib/XML/Grammar/FictionBase/FromProto/Parser/LineIterator.pm',
    'lib/XML/Grammar/FictionBase/FromProto/Parser/XmlIterator.pm',
    'lib/XML/Grammar/FictionBase/TagsTree2XML.pm',
    'lib/XML/Grammar/FictionBase/XSLT/Converter.pm',
    'lib/XML/Grammar/Screenplay.pm',
    'lib/XML/Grammar/Screenplay/App/FromProto.pm',
    'lib/XML/Grammar/Screenplay/App/ToDocBook.pm',
    'lib/XML/Grammar/Screenplay/App/ToHTML.pm',
    'lib/XML/Grammar/Screenplay/Base.pm',
    'lib/XML/Grammar/Screenplay/FromProto.pm',
    'lib/XML/Grammar/Screenplay/FromProto/Parser.pm',
    'lib/XML/Grammar/Screenplay/FromProto/Parser/QnD.pm',
    'lib/XML/Grammar/Screenplay/ToDocBook.pm',
    'lib/XML/Grammar/Screenplay/ToHTML.pm',
    'lib/XML/Grammar/Screenplay/ToTEI.pm',
    't/00-compile.t',
    't/00-libxml-basic.t',
    't/app-to-html.t',
    't/base/line-iterator-unit.t',
    't/fiction/data/docbook/a-href-with-id-and-lang.docbook.xml',
    't/fiction/data/docbook/sections-a-href.docbook.xml',
    't/fiction/data/docbook/sections-and-paras.docbook.xml',
    't/fiction/data/docbook/sections-p-b-i-comments.docbook.xml',
    't/fiction/data/docbook/with-blockquote-with-lang-and-id.docbook.xml',
    't/fiction/data/docbook/with-blockquote.docbook.xml',
    't/fiction/data/docbook/with-internal-description.docbook.xml',
    't/fiction/data/docbook/with-programlisting.docbook.xml',
    't/fiction/data/docbook/with-span.docbook.xml',
    't/fiction/data/docbook/with-ul-ol-li.docbook.xml',
    't/fiction/data/docbook/with-xml-lang-attr-in-section.docbook.xml',
    't/fiction/data/docbook/with-xml-lang-attribute.docbook.xml',
    't/fiction/data/proto-text-invalid/inner-desc-inside-char-addressing.txt',
    't/fiction/data/proto-text-invalid/leading-space-at-para-start.txt',
    't/fiction/data/proto-text-invalid/leading-space.txt',
    't/fiction/data/proto-text-invalid/no-right-angle.txt',
    't/fiction/data/proto-text-invalid/not-start-with-tag.txt',
    't/fiction/data/proto-text-invalid/wrong-close-tag.txt',
    't/fiction/data/proto-text-invalid/wrong-closing-inner-tag.txt',
    't/fiction/data/proto-text/a-href-with-id-and-lang.txt',
    't/fiction/data/proto-text/paras-with-entities-at-start-of-line.txt',
    't/fiction/data/proto-text/sections-a-href.txt',
    't/fiction/data/proto-text/sections-and-paras.txt',
    't/fiction/data/proto-text/sections-p-b-i-comments.txt',
    't/fiction/data/proto-text/sections-p-b-i.txt',
    't/fiction/data/proto-text/with-blockquote-with-lang-and-id.txt',
    't/fiction/data/proto-text/with-blockquote.txt',
    't/fiction/data/proto-text/with-comment-with-newlines.txt',
    't/fiction/data/proto-text/with-internal-description.txt',
    't/fiction/data/proto-text/with-programlisting.txt',
    't/fiction/data/proto-text/with-span.txt',
    't/fiction/data/proto-text/with-style-tag-at-start-of-paragraph.txt',
    't/fiction/data/proto-text/with-ul-ol-li.txt',
    't/fiction/data/proto-text/with-xml-lang-attr-in-section.txt',
    't/fiction/data/proto-text/with-xml-lang-attribute.txt',
    't/fiction/data/xml/a-href-with-id-and-lang.xml',
    't/fiction/data/xml/paras-with-entities-at-start-of-line.xml',
    't/fiction/data/xml/sections-a-href.xml',
    't/fiction/data/xml/sections-and-paras.xml',
    't/fiction/data/xml/sections-p-b-i-comments.xml',
    't/fiction/data/xml/sections-p-b-i.xml',
    't/fiction/data/xml/with-blockquote-with-lang-and-id.xml',
    't/fiction/data/xml/with-blockquote.xml',
    't/fiction/data/xml/with-comment-with-newlines.xml',
    't/fiction/data/xml/with-programlisting.xml',
    't/fiction/data/xml/with-span.xml',
    't/fiction/data/xml/with-style-tag-at-start-of-paragraph.xml',
    't/fiction/data/xml/with-ul-ol-li.xml',
    't/fiction/data/xml/with-xml-lang-attr-in-section.xml',
    't/fiction/data/xml/with-xml-lang-attribute.xml',
    't/fiction/proto-text-invalid.t',
    't/fiction/proto-text-to-xml-using-custom-parser.t',
    't/fiction/qnd-parser-unit.t',
    't/fiction/to-docbook.t',
    't/fiction/to-xhtml.t',
    't/lib/XmlGrammarTestXML.pm',
    't/lib/run-test-1/XML/Grammar/Fiction/ConfigData.pm',
    't/screenplay/app-to-html.t',
    't/screenplay/data/docbook/with-internal-description.docbook.xml',
    't/screenplay/data/proto-text-invalid/inner-desc-inside-char-addressing.txt',
    't/screenplay/data/proto-text-invalid/non-terminated-desc.txt',
    't/screenplay/data/proto-text/dialogue-with-several-paragraphs.txt',
    't/screenplay/data/proto-text/nested-s.txt',
    't/screenplay/data/proto-text/scenes-with-langs.txt',
    't/screenplay/data/proto-text/scenes-with-titles.txt',
    't/screenplay/data/proto-text/two-nested-s.txt',
    't/screenplay/data/proto-text/with-brs.txt',
    't/screenplay/data/proto-text/with-colon-inside-description.txt',
    't/screenplay/data/proto-text/with-comments-with-newlines.txt',
    't/screenplay/data/proto-text/with-comments.txt',
    't/screenplay/data/proto-text/with-description.txt',
    't/screenplay/data/proto-text/with-dialogue.txt',
    't/screenplay/data/proto-text/with-entities.txt',
    't/screenplay/data/proto-text/with-i-element-inside-paragraphs.txt',
    't/screenplay/data/proto-text/with-img-element-inside-paragraphs.txt',
    't/screenplay/data/proto-text/with-internal-description-at-start-of-line.txt',
    't/screenplay/data/proto-text/with-internal-description.txt',
    't/screenplay/data/proto-text/with-multi-line-comments.txt',
    't/screenplay/data/proto-text/with-multi-para-desc.txt',
    't/screenplay/data/proto-text/with-numeric-entities.txt',
    't/screenplay/data/proto-text/with-tags-inside-paragraphs.txt',
    't/screenplay/data/tei/dialogue-with-several-paragraphs.tei.xml',
    't/screenplay/data/tei/nested-s.tei.xml',
    't/screenplay/data/tei/scenes-with-titles.tei.xml',
    't/screenplay/data/tei/two-nested-s.tei.xml',
    't/screenplay/data/tei/with-brs.tei.xml',
    't/screenplay/data/tei/with-colon-inside-description.tei.xml',
    't/screenplay/data/tei/with-comments.tei.xml',
    't/screenplay/data/tei/with-description.tei.xml',
    't/screenplay/data/tei/with-dialogue.tei.xml',
    't/screenplay/data/tei/with-entities.tei.xml',
    't/screenplay/data/tei/with-i-element-inside-paragraphs.tei.xml',
    't/screenplay/data/tei/with-internal-description-at-start-of-line.tei.xml',
    't/screenplay/data/tei/with-internal-description.tei.xml',
    't/screenplay/data/tei/with-multi-line-comments.tei.xml',
    't/screenplay/data/tei/with-multi-para-desc.tei.xml',
    't/screenplay/data/tei/with-numeric-entities.tei.xml',
    't/screenplay/data/tei/with-tags-inside-paragraphs.tei.xml',
    't/screenplay/data/xml/dialogue-with-several-paragraphs.xml',
    't/screenplay/data/xml/nested-s.xml',
    't/screenplay/data/xml/scenes-with-langs.xml',
    't/screenplay/data/xml/scenes-with-titles.xml',
    't/screenplay/data/xml/two-nested-s.xml',
    't/screenplay/data/xml/with-brs.xml',
    't/screenplay/data/xml/with-colon-inside-description.xml',
    't/screenplay/data/xml/with-comments-with-newlines.xml',
    't/screenplay/data/xml/with-comments.xml',
    't/screenplay/data/xml/with-description.xml',
    't/screenplay/data/xml/with-dialogue.xml',
    't/screenplay/data/xml/with-entities.xml',
    't/screenplay/data/xml/with-i-element-inside-paragraphs.xml',
    't/screenplay/data/xml/with-img-element-inside-paragraphs.xml',
    't/screenplay/data/xml/with-internal-description-at-start-of-line.xml',
    't/screenplay/data/xml/with-internal-description.xml',
    't/screenplay/data/xml/with-multi-line-comments.xml',
    't/screenplay/data/xml/with-multi-para-desc.xml',
    't/screenplay/data/xml/with-numeric-entities.xml',
    't/screenplay/data/xml/with-tags-inside-paragraphs.xml',
    't/screenplay/lib/run-test-1/XML/Grammar/Screenplay/ConfigData.pm',
    't/screenplay/proto-text-invalid.t',
    't/screenplay/proto-text-to-xml-using-custom-parser.t',
    't/screenplay/qnd-parser-unit.t',
    't/screenplay/to-docbook.t',
    't/screenplay/to-tei.t',
    't/screenplay/to-xhtml.t'
);

notabs_ok($_) foreach @files;
done_testing;
